//-----------------------------------------------------------
// WAVEFLT2 
// Copyright (c) 1999-2002 Tomoya Tokairin
// LICENSE: GPL
//
// adaptive FIR filter

#include "filter.h"

double* ADP_coeff[2] = {NULL,NULL}; // filter cofficients (L-R)
DWORD	ADP_dwLength;	// length of filter
double* ADP_buffer[2] = {NULL,NULL}; // delay buffer (L-R)
DWORD	ADP_DwBufferSize; // size of delay buffer
DWORD	ADP_DwBufferPos[2] = {0,0};
double  ADP_kaiser[MAXFILTERPOINT]; // kaiser window



//------------------------------------
// get coefficients
DWORD EXPORT GetADPCoef(double* coef[2])
{

	coef[0] = ADP_coeff[0];
	coef[1] = ADP_coeff[1];

	return ADP_dwLength;
}




//-------------------------------------------------------------
// clear delay buffer
void EXPORT ClearADPBuf(){

	DWORD i;
	
	for(i=0;i<2;i++){
		if(ADP_buffer[i]) memset(ADP_buffer[i],0,sizeof(double)*ADP_DwBufferSize);
		ADP_DwBufferPos[i] = 0;
	}

}


//-------------------------------------------------------------
// clear cofficients
void ClearADPCoef(){

	DWORD i;
	
	for(i=0;i<2;i++){
		if(ADP_coeff[i]) memset(ADP_coeff[i],0,sizeof(double)*ADP_dwLength);
	}

}


//--------------------------------------------------------
// initialize ADP filter
void PrepareADP(DWORD dwADPleng,  // length
				double dDB,   // dB, loss
				double dbLevel, // trainging level
				DWORD dwDelayPoints, // points of delay
				DWORD dwChn
				){

	DWORD i;
	LONG ii;
	double alpha;
	double myu;  // correction factor
	
	ADP_dwLength = dwADPleng;
	for(i=0;i<dwChn;i++){
		if(ADP_coeff[i] == NULL) ADP_coeff[i]= (double*)malloc(sizeof(double)*ADP_dwLength+1024); 
		memset(ADP_coeff[i],0,sizeof(double)*ADP_dwLength); 
	}

	ADP_DwBufferSize = dwDelayPoints + dwADPleng;
	for(i=0;i<dwChn;i++){
		if(ADP_buffer[i] == NULL) ADP_buffer[i] = (double*)malloc(sizeof(double)*ADP_DwBufferSize +1024);
		memset(ADP_buffer[i],0,sizeof(double)*ADP_DwBufferSize);
		ADP_DwBufferPos[i] = 0;
	}

	myu = 2.* 1./ADP_dwLength * dbLevel;

	// set kaiser window value
	// kaiser window value is multiplied by correction factor here in advance.
	alpha = kaiserAlpha(dDB);
	for(ii=-(LONG)ADP_dwLength/2; ii<(LONG)ADP_dwLength/2+1; ii++)
		ADP_kaiser[ii+ADP_dwLength/2] = myu * I0(alpha*sqrt(1-(2*ii/(double)(ADP_dwLength-1)) * (2*ii/(double)(ADP_dwLength-1)) ))/I0(alpha);

	ClearADPBuf();
	ClearADPCoef();
}



//---------------------------------------------------------------
// unprepare
void unprepareADP(){

	DWORD i;

	for(i=0;i<2;i++) {
		if(ADP_buffer[i]) free(ADP_buffer[i]);
		if(ADP_coeff[i]) free(ADP_coeff[i]);
		ADP_buffer[i] = NULL;
		ADP_coeff[i] = NULL;
	}
}




//--------------------------------------------------------
// ADP LMS filter
void ADPLMS(double* lpFilterBuf, // buffer
			DWORD dwPointsInBuf, // points
			DWORD dwChn
			){
	
	DWORD i,i2,dwBufferPos; 
	double dFoo;
	double *buf1,*buf2;

	dwBufferPos = ADP_DwBufferPos[dwChn];

	SetConvSize(ADP_dwLength);
	buf1 = GetConvBufA();
	buf2 = GetConvBufB();

	// copy data to convolution buffer
	memcpy(buf2,ADP_coeff[dwChn],sizeof(double)*ADP_dwLength);
	for(i2 = 0; i2 < ADP_dwLength; i2++) buf1[i2] = ADP_buffer[dwChn][(dwBufferPos+i2) % ADP_DwBufferSize];

	for(i=0;i< dwPointsInBuf;i++){

		ADP_buffer[dwChn][dwBufferPos] = lpFilterBuf[i];
		
		dFoo = CONV();
		
		// output = e(k)
		lpFilterBuf[i] -= dFoo; 

		// LMS
		dFoo = lpFilterBuf[i];
		for(i2 = 0; i2 < ADP_dwLength; i2++) buf2[i2] += (ADP_kaiser[i2] * dFoo * buf1[i2]);
		
		dwBufferPos = (dwBufferPos+1) % ADP_DwBufferSize;
		memmove(buf1,buf1+1,sizeof(double)*(ADP_dwLength-1));
		buf1[ADP_dwLength-1] = ADP_buffer[dwChn][(dwBufferPos+(ADP_dwLength-1)) % ADP_DwBufferSize];
	}

	memcpy(ADP_coeff[dwChn],buf2,sizeof(double)*ADP_dwLength);
	ADP_DwBufferPos[dwChn] = dwBufferPos;

}


//EOF