//-----------------------------------------------------------
// WAVEFLT2 
// Copyright (c) 1999-2002 Tomoya Tokairin
// LICENSE: GPL
// 
// MDCT ,IMDCT

#include "filter.h"

#define MDCT_NUM	4


COMPLEX *MDCT_CompBuf[MDCT_NUM];
double *MDCT_input[MDCT_NUM];
DWORD MDCT_dwLn[MDCT_NUM];  // length


//------------------------
// prepare
void prepareMDCT(DWORD N,DWORD dwMDCTnum,DWORD dwFFTnum){

	if(!MDCT_CompBuf[dwMDCTnum]) MDCT_CompBuf[dwMDCTnum] = (LPCOMPLEX)malloc(sizeof(COMPLEX)*N/4 +1024);
	if(!MDCT_input[dwMDCTnum]) MDCT_input[dwMDCTnum] = (double*)malloc(sizeof(double)*N +1024);
	MDCT_dwLn[dwMDCTnum] = N;

	prepareFft(N/4,dwFFTnum); 
}



//------------------------
// unprepare
void unprepareMDCT(){

	DWORD i;
	
	for(i=0;i<MDCT_NUM;i++){
		if(MDCT_CompBuf[i]) free(MDCT_CompBuf[i]);
		if(MDCT_input[i]) free(MDCT_input[i]);
		MDCT_CompBuf[i] = NULL;
		MDCT_input[i] = NULL;
	}
}



//---------------------------
// MDCT  (Bosse's method) 
void MDCT_fast(double* x, // N	
		  double* f,	// N/2	
		  DWORD dwMDCTnum,
		  DWORD dwFFTnum
		  ){
	
	double theta,foo,foo1,foo2;
	DWORD i,N;

	N = MDCT_dwLn[dwMDCTnum];

	// shift
	for(i = 0; i < N/4; i++) MDCT_input[dwMDCTnum][i] = -x[i+N/4*3];
	for(i = N/4; i < N; i++) MDCT_input[dwMDCTnum][i] = x[i-N/4];
	
	// pre rotation
	for(i = 0; i < N/4; i++)
    { 
		foo1 = MDCT_input[dwMDCTnum][2*i] - MDCT_input[dwMDCTnum][N-1-2*i];
		foo2 = -(MDCT_input[dwMDCTnum][N/2+2*i] - MDCT_input[dwMDCTnum][N/2-1-2*i]);
		theta = 2.*PI*(i+1./8.)/N;
		MDCT_CompBuf[dwMDCTnum][i].r =  foo1 * cos(theta) + foo2 * sin(theta);
		MDCT_CompBuf[dwMDCTnum][i].i = -foo1 * sin(theta) + foo2 * cos(theta);
    }

	calcFftFast(MDCT_CompBuf[dwMDCTnum],dwFFTnum);

	// post rotation
	foo = 2./sqrt(N);  
	for(i = 0; i < N/4; i++)
    {
		foo1 = MDCT_CompBuf[dwMDCTnum][i].r;
		foo2 = MDCT_CompBuf[dwMDCTnum][i].i;
		theta = 2.*PI*(i+1./8.)/N;
		MDCT_CompBuf[dwMDCTnum][i].r = foo *( foo1 * cos(theta) + foo2 * sin(theta));
		MDCT_CompBuf[dwMDCTnum][i].i = foo *(-foo1 * sin(theta) + foo2 * cos(theta));
    }
	
	// sorting
	for(i = 0; i < N/4; i++)
    {
		f[2*i] = MDCT_CompBuf[dwMDCTnum][i].r;
		f[N/2-1-2*i] = -MDCT_CompBuf[dwMDCTnum][i].i; 
    }
}



//---------------------------
// IMDCT
void IMDCT_fast(double* x,	// N 
				double* f,	// N/2 
				DWORD dwMDCTnum,
				DWORD dwFFTnum
				){
	
	double theta;
	double foo,foo1,foo2;
	DWORD i,N;

	N = MDCT_dwLn[dwMDCTnum];
	
	// pre rotation
	for(i = 0; i < N/4; i++)
    {
		foo1 = f[2*i];
		foo2 = f[N/2-1-2*i];
		theta = 2.*PI*(i+1./8.)/N;
		MDCT_CompBuf[dwMDCTnum][i].r =  foo1 * cos(theta) + foo2 * sin(theta);
		MDCT_CompBuf[dwMDCTnum][i].i = -foo1 * sin(theta) + foo2 * cos(theta);
    }
	
	calcFftFast(MDCT_CompBuf[dwMDCTnum],dwFFTnum);
	
	// post rotation
	foo = 2./sqrt(N);
	for(i = 0; i < N/4; i++)
    {
		foo1 = MDCT_CompBuf[dwMDCTnum][i].r;
		foo2 = MDCT_CompBuf[dwMDCTnum][i].i;
		theta = 2.*PI*(i+1./8.0)/N;
		MDCT_CompBuf[dwMDCTnum][i].r = foo *( foo1 * cos(theta) + foo2 * sin(theta));
		MDCT_CompBuf[dwMDCTnum][i].i = foo *(-foo1 * sin(theta) + foo2 * cos(theta));
    }
	
	// sorting
	for(i = 0; i < N/4; i++)
    {
		MDCT_input[dwMDCTnum][2*i] = MDCT_CompBuf[dwMDCTnum][i].r;
		MDCT_input[dwMDCTnum][N/2+2*i] = MDCT_CompBuf[dwMDCTnum][i].i; 
    }
	for(i = 1; i < N; i+=2)
    {
		MDCT_input[dwMDCTnum][i] = -MDCT_input[dwMDCTnum][N-1-i];
    }
	
	// shift
	for(i = 0;i < N/4*3; i++) x[i] = MDCT_input[dwMDCTnum][i+N/4];
	for(i = N/4*3; i < N; i++) x[i] = -MDCT_input[dwMDCTnum][i-3*N/4];
}


// EOF
